diff --git a/common/common.c b/common/common.c
index d03201d..ca11304 100644
--- a/common/common.c
+++ b/common/common.c
@@ -952,9 +952,17 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
     OPT("ratetol")
         p->rc.f_rate_tolerance = !strncmp("inf", value, 3) ? 1e9 : atof(value);
     OPT("vbv-maxrate")
-        p->rc.i_vbv_max_bitrate = atoi(value);
+    {
+        double d = atof(value);
+        p->rc.i_nal_hrd_max_bitrate = x264_clip3f(d * 1000, 0, 2000000000);
+        p->rc.i_vbv_max_bitrate = (p->rc.i_nal_hrd_max_bitrate + 999) / 1000;
+    }
     OPT("vbv-bufsize")
-        p->rc.i_vbv_buffer_size = atoi(value);
+    {
+        double d = atof(value);
+        p->rc.i_nal_hrd_buffer_size = x264_clip3f(d * 1000, 0, 2000000000);
+        p->rc.i_vbv_buffer_size = (p->rc.i_nal_hrd_buffer_size + 999) / 1000;
+    }
     OPT("vbv-init")
         p->rc.f_vbv_buffer_init = atof(value);
     OPT2("ipratio", "ip-factor")
diff --git a/encoder/encoder.c b/encoder/encoder.c
index 7fb3972..3083b35 100644
--- a/encoder/encoder.c
+++ b/encoder/encoder.c
@@ -634,6 +634,8 @@ static int x264_validate_parameters( x264_t *h, int b_open )
         x264_log( h, X264_LOG_WARNING, "VBV maxrate specified, but no bufsize, ignored\n" );
         h->param.rc.i_vbv_max_bitrate = 0;
     }
+    h->param.rc.i_nal_hrd_buffer_size = x264_clip3( h->param.rc.i_nal_hrd_buffer_size, 0, h->param.rc.i_vbv_buffer_size * 1000 );
+    h->param.rc.i_nal_hrd_max_bitrate = x264_clip3( h->param.rc.i_nal_hrd_max_bitrate, 0, h->param.rc.i_vbv_max_bitrate * 1000 );
 
     h->param.i_slice_max_size = X264_MAX( h->param.i_slice_max_size, 0 );
     h->param.i_slice_max_mbs = X264_MAX( h->param.i_slice_max_mbs, 0 );
diff --git a/encoder/ratecontrol.c b/encoder/ratecontrol.c
index 0437fbd..dfc8671 100644
--- a/encoder/ratecontrol.c
+++ b/encoder/ratecontrol.c
@@ -524,8 +524,12 @@ void x264_ratecontrol_init_reconfigurable( x264_t *h, int b_init )
             #define BR_SHIFT  6
             #define CPB_SHIFT 4
 
-            int bitrate = 1000*h->param.rc.i_vbv_max_bitrate;
-            int bufsize = 1000*h->param.rc.i_vbv_buffer_size;
+            int bitrate = h->param.rc.i_nal_hrd_max_bitrate;
+            if( (bitrate + 999) / 1000 != h->param.rc.i_vbv_max_bitrate )
+                bitrate = h->param.rc.i_vbv_max_bitrate * 1000;
+            int bufsize = h->param.rc.i_nal_hrd_buffer_size;
+            if( (bufsize + 999) / 1000 != h->param.rc.i_vbv_buffer_size )
+                bufsize = h->param.rc.i_vbv_buffer_size * 1000;
 
             // normalize HRD size and rate to the value / scale notation
             h->sps->vui.hrd.i_bit_rate_scale = x264_clip3( x264_ctz( bitrate ) - BR_SHIFT, 0, 15 );
diff --git a/x264.h b/x264.h
index 9219114..715ec52 100644
--- a/x264.h
+++ b/x264.h
@@ -388,6 +388,8 @@ typedef struct x264_param_t
         int         i_vbv_max_bitrate;
         int         i_vbv_buffer_size;
         float       f_vbv_buffer_init; /* <=1: fraction of buffer_size. >1: kbit */
+        int         i_nal_hrd_max_bitrate; /* i_vbv_max_bitrate with bit precision instead of kbit */
+        int         i_nal_hrd_buffer_size; /* i_vbv_buffer_size with bit precision instead of kbit */
         float       f_ip_factor;
         float       f_pb_factor;
 